import { Message, MessageType } from '../i18n/definitions';
import { Failure, Result, Success } from './result';
export type ResultAsyncValue<S> = PromiseLike<Success<S>>;
export interface ResultAsyncHelpers {
    liftResult<S>(result: Result<S>): ResultAsyncValue<S>;
    liftSuccess<S>(value: S): ResultAsyncValue<S>;
    liftFailure(code: string, parameters?: Record<string, string>, issuer?: string, type?: MessageType): ResultAsyncValue<never>;
    fromResultPromise<S>(promise: PromiseLike<Result<S>>): ResultAsyncValue<S>;
    fromPromise<S>(promise: PromiseLike<S>, failure: (reason: unknown) => Failure): ResultAsyncValue<S>;
}
export interface ResultAsync<S> extends PromiseLike<Result<S>> {
    run(): Promise<Result<S>>;
    observe(on: (result: Result<S>) => void): ResultAsync<S>;
    mapSuccess<S2>(onSuccess: (value: S) => Success<S2>): ResultAsync<S2>;
    mapFailure(onFailure: (errors: Message[]) => Failure): ResultAsync<S>;
    mapBoth<S2>(onFailure: (errors: Message[]) => Failure, onSuccess: (value: S) => Success<S2>): ResultAsync<S2>;
    chain<S2>(next: (result: Result<S>) => PromiseLike<Result<S2>>): ResultAsync<S2>;
    chainSuccess<S2>(next: (value: S) => PromiseLike<Result<S2>>): ResultAsync<S2>;
    chainFailure<S2>(next: (errors: Message[]) => PromiseLike<Result<S2>>): ResultAsync<S | S2>;
    chainBoth<S2, S3>(nextFailure: (errors: Message[]) => PromiseLike<Result<S2>>, nextSuccess: (value: S) => PromiseLike<Result<S3>>): ResultAsync<S2 | S3>;
}
interface ResultAsyncTypeRef {
    <S>(runPromise: (helpers: ResultAsyncHelpers) => PromiseLike<Success<S>>): ResultAsync<S>;
    liftResult<S>(result: Result<S>): ResultAsync<S>;
    liftSuccess<S>(value: S): ResultAsync<S>;
    liftFailure(code: string, parameters?: Record<string, string>, issuer?: string, type?: MessageType): ResultAsync<never>;
    check<S>(condition: boolean, success: S, failure: Failure): ResultAsync<S>;
    if<S1, S2>(condition: boolean, ifResult: Result<S1>, elseResult: Result<S2>): ResultAsync<S1 | S2>;
    fromResultPromise<S>(promise: () => PromiseLike<Result<S>>): ResultAsync<S>;
    fromPromise<S>(promise: () => PromiseLike<S>, failure: (reason: unknown) => Failure): ResultAsync<S>;
    allMustSuccess<S>(list: ResultAsync<S>[]): ResultAsync<S[]>;
    someMayFail<S>(list: ResultAsync<S>[]): ResultAsync<S[]>;
    all<S>(list: ResultAsync<S>[]): ResultAsync<S[]>;
    first<S>(list: S[], condition: (value: S) => ResultAsync<boolean>): ResultAsync<S | undefined>;
}
export declare const ResultAsync: ResultAsyncTypeRef;
export {};
