"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultAsync = void 0;
const definitions_1 = require("../i18n/definitions");
const result_1 = require("./result");
const resultAsyncHelpers = {
    liftResult(result) {
        if (result.isSuccess()) {
            return Promise.resolve(result);
        }
        throw result;
    },
    liftSuccess(value) {
        return Promise.resolve(new result_1.Success(value));
    },
    liftFailure(code, parameters, issuer, type = definitions_1.MessageType.ProcessError) {
        throw result_1.Failure.single(code, parameters, issuer, type);
    },
    fromResultPromise(promise) {
        return promise.then(resultAsyncHelpers.liftResult);
    },
    fromPromise(promise, failure) {
        return promise.then((result) => resultAsyncHelpers.liftResult(new result_1.Success(result)), (reason) => resultAsyncHelpers.liftResult(typeof failure === 'function' ? failure(reason) : failure));
    },
};
class ResultAsyncImpl {
    constructor(runPromise) {
        this.runPromise = runPromise;
        this.then = (onfulfilled, onrejected) => this.run().then(onfulfilled, onrejected);
    }
    run() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.runPromise(resultAsyncHelpers);
            }
            catch (e) {
                if (e instanceof Error) {
                    return result_1.Failure.single('catchedError', { name: e.name, message: e.message, stack: (_a = e.stack) !== null && _a !== void 0 ? _a : '' }, 'lapala', definitions_1.MessageType.TechnicalIssue);
                }
                else if (e instanceof result_1.Failure) {
                    return e;
                }
                return result_1.Failure.single('unknownErrorType', { error: JSON.stringify(e) }, 'lapala', definitions_1.MessageType.TechnicalIssue);
            }
        });
    }
    observe(f) {
        return (0, exports.ResultAsync)(({ liftResult }) => __awaiter(this, void 0, void 0, function* () { return liftResult((yield this.run()).observe(f)); }));
    }
    mapBoth(onFailure, onSuccess) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            const result = yield this.run();
            return helpers.liftResult(result.mapBoth(onFailure, onSuccess));
        }));
    }
    mapSuccess(onSuccess) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            const result = yield this.run();
            return helpers.liftResult(result.mapSuccess(onSuccess));
        }));
    }
    mapFailure(onFailure) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.runPromise(helpers);
            }
            catch (e) {
                const failure = e;
                throw onFailure(failure.errors).addWarnings(failure.warnings);
            }
        }));
    }
    chain(next) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            const result = yield this.run();
            return helpers.fromResultPromise(next(result).then((nextResult) => nextResult.addWarnings(nextResult.warnings)));
        }));
    }
    chainSuccess(next) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            const success = yield this.runPromise(helpers);
            return helpers.fromResultPromise(next(success.value).then((result) => result.addWarnings(success.warnings)));
        }));
    }
    chainFailure(next) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.runPromise(helpers);
            }
            catch (e) {
                const failure = e;
                return helpers.fromResultPromise(next(failure.errors).then((success) => success.addWarnings(failure.warnings)));
            }
        }));
    }
    chainBoth(nextFailure, nextSuccess) {
        return (0, exports.ResultAsync)((helpers) => __awaiter(this, void 0, void 0, function* () {
            try {
                const success = yield this.runPromise(helpers);
                return yield helpers.fromResultPromise(nextSuccess(success.value).then((result) => result.addWarnings(success.warnings)));
            }
            catch (e) {
                const failure = e;
                return helpers.fromResultPromise(nextFailure(failure.errors).then((success) => success.addWarnings(failure.warnings)));
            }
        }));
    }
}
exports.ResultAsync = Object.assign((runPromise) => new ResultAsyncImpl(runPromise), {
    liftResult: (result) => (0, exports.ResultAsync)(({ liftResult }) => liftResult(result)),
    liftSuccess: (value) => (0, exports.ResultAsync)(({ liftResult }) => liftResult(new result_1.Success(value))),
    liftFailure: (code, parameters, issuer, type) => (0, exports.ResultAsync)(({ liftFailure }) => liftFailure(code, parameters, issuer, type)),
    check: (condition, success, failure) => exports.ResultAsync.liftResult(condition ? new result_1.Success(success) : failure),
    if: (condition, ifResult, elseResult) => condition ? exports.ResultAsync.liftResult(ifResult) : exports.ResultAsync.liftResult(elseResult),
    fromResultPromise: (promise) => (0, exports.ResultAsync)(({ fromResultPromise }) => fromResultPromise(promise())),
    fromPromise: (promise, failure) => (0, exports.ResultAsync)(({ fromPromise }) => fromPromise(promise(), failure)),
    allMustSuccess: (list) => exports.ResultAsync.fromResultPromise(() => Promise.all(list).then(result_1.Result.sequence)),
    someMayFail: (list) => exports.ResultAsync.fromResultPromise(() => Promise.all(list).then(result_1.Result.sequenceWithoutFailure)),
    all: (list) => exports.ResultAsync.fromResultPromise(() => Promise.all(list).then(result_1.Result.sequenceWithAllFailures)),
    first: (list, condition) => (0, exports.ResultAsync)(({ liftSuccess, liftResult }) => __awaiter(void 0, void 0, void 0, function* () {
        for (const item of list) {
            const result = yield condition(item);
            if (result.isFailure()) {
                return liftResult(result);
            }
            else if (result.value) {
                return liftSuccess(item);
            }
        }
        return liftSuccess(undefined);
    })),
});
ResultAsyncImpl.prototype.constructor = exports.ResultAsync;
